/*
 * Copyright (c) 1997-2000 Akihiko YAMAMOTO (star@eos.hokudai.ac.jp)
 * All rights reserved.
 *
 * Header files of Password-check module for ProFTPD
 * $Id: pwcheck.h,v 1.24 2000/09/06 02:05:03 star Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/* 
 * Change the following path definitions to suit your needs. 
 */
#define	_PATH_PWDERRMSG		"/usr/local/etc/msg.Invalid_PW"
#define	_PATH_DISPLAYLOGIN	"/usr/local/etc/msg.display_login"
#define	_PATH_INVALIDPWCLASS	"/usr/local/etc/Invalid_PW_class"
#define	_PATH_INVALIDPWHOST	"/usr/local/etc/Invalid_PW_host"
#define	_PATH_SERVERDEADCLASS	"/usr/local/etc/msg.serverdead_class"
#define	_PATH_SERVERDEADHOST	"/usr/local/etc/msg.serverdead_host"
#define	_PATH_REVERSEDNSCLASS	"/usr/local/etc/msg.reverseDNS_class"
#define	_PATH_REVERSEDNSHOST	"/usr/local/etc/msg.reverseDNS_host"
#define	_PATH_MATCHDNSCLASS	"/usr/local/etc/msg.matchDNS_class"
#define	_PATH_MATCHDNSHOST	"/usr/local/etc/msg.matchDNS_host"
#define	_PATH_GOAWAYCLASS	"/usr/local/etc/msg.goaway_class"
#define	_PATH_GOAWAYHOST	"/usr/local/etc/msg.goaway_host"
#define	_PATH_MSGPREFIX		"/usr/local/etc/prefix"



/*
 * =================================
 * DO NOT TOUCH ANYTHING BELOW HERE!
 * =================================
 */



/* 
 * Path definition for dump parameters 
 * This can be overridden by a named file specified by "PCKdump".
 *
 * Note that the owner and group of this file will be "User.Group"
 * which is defined by "User" and "Group" directives, respectively.
 * Normaly, these are: "nobody.nogroup" by default.
 */
#define	_PATH_DUMP		"/tmp/.__pck.dump.pck__"

#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>

#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>

#include <sys/socket.h>
#ifdef SYSSYSLOG
#include <sys/syslog.h>
#else
#include <syslog.h>
#endif

#include <time.h>
#include <sys/param.h>
#include <sys/types.h>

enum JUDGE { Not_Valid, Valid };

typedef struct _CHECK *CHECK_PTR;

typedef struct _CHECK
{
  int		host;
  int		class;
  int		host_override;
  int		class_override;
  const char	*host_path;
  const char	*class_path;
  CHECK_PTR	ptr;
} CHECK;

typedef struct _REMOTE *REMOTE_PTR;

typedef struct _REMOTE
{
  const char	*name;
  const char	*ip;
  const char	*anonname;
  const char	*pass;
  const char	*class;
  int		class_defined;
  REMOTE_PTR	ptr;
} REMOTE;


typedef struct _LOCAL *LOCAL_PTR;

typedef struct _LOCAL
{
  char		name	[MAXHOSTNAMELEN];
  const char	*ip;
  const char	*cname;
  const char	*name_server;
  LOCAL_PTR	ptr;
} LOCAL;

typedef struct _PATH *PATH_PTR;

typedef struct _PATH
{
  const char	*dump;
  const char	*errmsg;
  const char	*invalidpw;
  const char	*display_login;
  const char	*msg_prefix;
  PATH_PTR	ptr;
} PATH;

typedef struct _PCK *PCK_PTR;

typedef struct _PCK
{
  /* structure CHECK */
  CHECK		server_dead;
  CHECK		goaway;
  CHECK		reverse_dns;
  CHECK		matching_dns;
  CHECK		pwcheck;

  /* structure REMOTE */
  REMOTE	remote;

  /* structure LOCAL */
  LOCAL		local;

  /* structure PATH */
  PATH		path;

  int		enable;
  int		valid;
  int		debug;
  int		dot_check;
  int		no_more_check;
  int		display_login;
  int		msg_prefix;
  int		dump;
  const char	*index;
  const char	*denymsg;
  const char	*grantmsg;
  char		check_mode	[5][MAXPATHLEN];
  char		check_order	[5][MAXPATHLEN];
  PCK_PTR	ptr;
} PCK;

#define  YesNo(x)	(((x))? "TRUE" : "FALSE")

#define  IsDigitAd(address,index)	\
		((pck_isdigitad2((address)) && !strcasecmp((index), "host")))

#define  NEW(T, c)		\
		((T *)malloc((unsigned int)(sizeof (T) * (c))))

#define  RENEW(p, T, c)		\
		(p = (T *)xrealloc((char *)(p), (unsigned int)(sizeof (T) * (c))))

/* Dispose allocated memory */
#define		pck_dispose(ptr)	((void) pck_xfree((void *)ptr))

#ifndef __BIT_TYPES_DEFINED__
typedef unsigned int            u_int32_t;
#endif  /* __BIT_TYPES_DEFINED__ */

#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN    64
#endif

#ifndef DONT_REMOVE
#define DONT_REMOVE        0
#endif

#ifndef REMOVE_CONTROL
#define REMOVE_CONTROL  0x01
#endif

#ifndef REMOVE_BRACKET
#define REMOVE_BRACKET  0x02
#endif

#ifndef FNM_PERIOD2
#define FNM_PERIOD2     0x08
#endif

#ifndef  NO_NEED_TO_CHECK
#define  NO_NEED_TO_CHECK  0   /* ignore checks for specified classes */
#endif

#ifndef  NEED_TO_CHECK
#define  NEED_TO_CHECK     1   /* do checks for specified classes */
#endif

#ifndef RECORD_NOT_FOUND
#define RECORD_NOT_FOUND   0   /* set when DNS record not found */
#endif

#ifndef RECORD_FOUND
#define RECORD_FOUND       1   /* set when DNS record found */
#endif

#ifndef REJECT
#define REJECT             0   /* flag to reject password */
#endif

#ifndef ACCEPT
#define ACCEPT             1   /* flag to accept password */
#endif

#ifndef NO_MORE_CHECK
#define NO_MORE_CHECK      2   /* skip any more checks */
#endif

#ifndef CHECKED_ONCE_FILE
#define CHECKED_ONCE_FILE  4   /* set when file check done */
#endif

#ifndef CHECKED_ONCE_DNS
#define CHECKED_ONCE_DNS   8   /* set when dns check done */
#endif

#ifndef NONE_DEFINED
#define NONE_DEFINED      16   /* flag for initialization */
#endif

#ifndef YESNO
#define YESNO             32   /* YES/NO flag for DNS checks */
#endif

#ifndef DNS_PATH_DEFINED
#define DNS_PATH_DEFINED  64   /* file path for DNS error messages */
#endif

#ifndef OVERRIDE_DEFINED
#define OVERRIDE_DEFINED 128   /* file path for DNS error messages */
#endif

#ifndef PCK_DEBUG_NONE
#define PCK_DEBUG_NONE         0   /* Syslog debug level */
#endif

#ifndef PCK_DEBUG_NORMAL
#define PCK_DEBUG_NORMAL       1   /* Syslog debug level */
#endif

#ifndef PCK_DEBUG_HIGH
#define PCK_DEBUG_HIGH         2   /* Syslog debug level */
#endif

#ifndef PCK_DEBUG_FULL
#define PCK_DEBUG_FULL         4   /* Syslog debug level */
#endif

#include <stdarg.h>

#define USE_REPLY_NOTFMT      (1<<1)	/* fmt is not a printf fmt (KLUDGE) */
#define USE_REPLY_LONG        (1<<2)	/* this is a long reply; use a - */

#include <arpa/inet.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>

#ifndef __CONF_H
#define __CONF_H

#include "config.h"

#ifdef HAVE_GLOB
# include <glob.h>
#endif

#ifdef HAVE_UTMPX_H
# include <utmpx.h>
#endif

#ifndef __PROFTPD_SUPPORT_LIBRARY

typedef struct in_addr p_in_addr_t;

#include "conf.h"

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "pool.h"
#include "proftpd.h"
#include "support.h"
#include "sets.h"
#include "dirtree.h"
#include "io.h"
#include "modules.h"
#include "inet.h"
#include "libsupp.h"
#include "fs.h"
#include "privs.h"
/*#include "ftp.h"*/
#define C_USER	"USER"		/* Specify a username */
#define C_PASS	"PASS"		/* Specify a password */
#define R_230	"230"		/* User logged in, proceed */
#define R_530	"530"		/* Not logged in */
#define R_DUP	NULL		/* Duplicate last numeric in ml response */
#define G_NONE	NULL
#include <grp.h>
#include <pwd.h>

#ifdef HAVE_FCNTL_H
# include <fcntl.h>
#endif

#endif

#endif
